@@ -26,8 +26,6 @@ import gobject
 import pango
 import os
 import logging
-import csv
-import ConfigParser
 
 import metarace
 from metarace import tod
@@ -40,9 +38,10 @@ from metarace import strops
 from metarace import uiutil
 from metarace import timerpane
 from metarace import printing
+from metarace import jsonconfig
 
 # rider commands
-RIDER_COMMANDS_ORD = [ 'add', 'del', 'que', 'dns', 'hd',
+RIDER_COMMANDS_ORD = [ 'add', 'del', 'que', 'onc', 'dns', 'hd',
                    'dnf', 'dsq', 'com', '']
 RIDER_COMMANDS = {'dns':'Did not start',
                    'dnf':'Did not finish',
@@ -52,6 +51,7 @@ RIDER_COMMANDS = {'dns':'Did not start',
                    'com':'Add comment',
                    'hd':'Outside time limit',
                    'dsq':'Disqualify',
+                   'onc':'Riders on course',
                    '':'',
                    }
 
@@ -83,6 +83,7 @@ key_starters = 'F3'                  # show next few starters in scratchpad
 key_armsync = 'F1'                   # arm for clock sync start
 key_armstart = 'F5'                  # arm for start impulse
 key_armfinish = 'F9'                 # arm for finish impulse
+key_raceover = 'F10'                 # flag race completion/not provisional
 
 # extended function key mappings
 key_reset = 'F5'                     # + ctrl for clear/abort
@@ -92,7 +93,7 @@ key_abort_finish = 'F8'		     # + ctrl abort B
 key_undo = 'Z'
 
 # config version string
-EVENT_ID = 'roadtt-1.2'
+EVENT_ID = u'roadtt-2.0'
 
 def sort_tally(x, y):
     """Points tally rough sort for stage tally."""
@@ -132,6 +133,9 @@ class irtt(object):
                     self.meet.scratch_clear()
                     glib.idle_add(self.delayed_announce)
                     return True
+                elif key == key_raceover:
+                    self.set_finished()
+                    return True
                 elif key == key_results:
                     #self.showresults()
                     return True
@@ -150,6 +154,16 @@ class irtt(object):
         #uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Idle')
         #self.log.info('Reset to IDLE')
 
+    def set_finished(self):
+        """Update event status to finished."""
+        if self.timerstat == 'finished':
+            self.timerstat = 'running'
+            uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Running')
+        else:
+            self.timerstat = 'finished'
+            uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Finished')
+            self.meet.stat_but.set_sensitive(False) 
+
     def armfinish(self):
         if self.timerstat == 'running':
             if self.fl.getstatus() != 'finish' and self.fl.getstatus() != 'armfin':
@@ -289,6 +303,11 @@ class irtt(object):
                 bib, ser = strops.bibstr2bibser(bibstr)
                 self.addrider(bib, ser)
             return True
+        elif acode == 'onc':
+            rlist = strops.reformat_bibserlist(rlist)
+            for bibstr in rlist.split():
+                self.add_starter(bibstr)
+            return True
         elif acode == 'dnf':
             self.dnfriders(strops.reformat_bibserlist(rlist))
             return True
@@ -334,156 +353,154 @@ class irtt(object):
     def loadconfig(self):
         """Load race config from disk."""
         self.riders.clear()
-        self.results = {'':tod.todlist('None')}
+        self.results = {u'':tod.todlist(u'UNCAT')}
         self.cats = []
 
-        # failsafe defaults -> dual timer, C0 start, PA/PB
-        # start is 0
-        # type specific overrides
+        cr = jsonconfig.config({u'event':{
+                u'startlist':u'',
+                u'id':EVENT_ID,
+                u'start':u'0',
+                u'comment':[],
+                                        u'categories':[],
+                                        u'lstart':u'0',
+                                        u'startgap':u'1:00',
+                                        u'remote_host':u'',
+                                        u'remote_channel':u'',
+                                        u'intermeds':[],
+                                        u'contests':[],
+                                        u'tallys':[],
+                                        u'finished':False,
+                                        u'recentstarts':[],
+                                       }})
+        cr.add_section(u'event')
+        cr.add_section(u'riders')
 
-        cr = ConfigParser.ConfigParser({'startlist':'',
-                                        'id':EVENT_ID,
-					'start':'0',
-                                        'comment':'',
-                                        'categories':'',
-                                        'lstart':'0',
-                                        'startgap':'1:00',
-                                        'remote_host':'',
-                                        'remote_channel':'',
-                                        'intermeds':'',
-                                        'contests':'',
-                                        'tallys':'',
-                                        'recentstarts':'',
-                                       })
-        cr.add_section('event')
-        cr.add_section('riders')
-        if os.path.isfile(self.configpath):
-            self.log.debug('Attempting to read config from '
-                               + repr(self.configpath))
-            cr.read(self.configpath)
+        # check for config file
+        try:
+            with open(self.configpath, 'rb') as f:
+                cr.read(f)
+        except Exception as e:
+            self.log.error(u'Reading event config: ' + unicode(e))
 
         # load default gap
-        self.startgap = tod.str2tod(cr.get('event', 'startgap'))
+        self.startgap = tod.str2tod(cr.get(u'event', u'startgap'))
         if self.startgap is None:
             self.startgap = tod.tod('1:00')
         
         # load _result_ categories
-        catlist = cr.get('event', 'categories')
-        if catlist == 'AUTO':
+        catlist = cr.get(u'event', u'categories')
+        if u'AUTO' in catlist:  # ignore any others and re-load from rdb
             self.cats = self.meet.rdb.listcats()
             self.autocats = True
             for cat in self.cats:
                 self.results[cat] = tod.todlist(cat)
         else:
             self.autocats = False
-            for cat in catlist.split(','):
-                if cat != '':
+            for cat in catlist:
+                if cat != u'':
                     cat = cat.upper()
                     self.cats.append(cat)
                     self.results[cat] = tod.todlist(cat)
-        self.cats.append('')    # always include the 'empty' cat
+        self.cats.append(u'')   # always include one empty cat
 
         # restore intermediates
-        for i in cr.get('event', 'intermeds').translate(
-                            strops.BIBLIST_TRANS).split():
+        for i in cr.get(u'event', u'intermeds'):
             if i in RESERVED_SOURCES:
-                self.log.info('Ignoring reserved intermediate: ' + repr(i))
+                self.log.info(u'Ignoring reserved intermediate: ' + repr(i))
             else:
-                crkey = 'intermed_' + i
-                descr = ''
-                places = ''
-                if cr.has_option(crkey, 'descr'):
-                    descr = cr.get(crkey, 'descr')
-                if cr.has_option(crkey, 'places'):
+                crkey = u'intermed_' + i
+                descr = u''
+                places = u''
+                if cr.has_option(crkey, u'descr'):
+                    descr = cr.get(crkey, u'descr')
+                if cr.has_option(crkey, u'places'):
                     places = strops.reformat_placelist(
-                                 cr.get(crkey, 'places'))
+                                 cr.get(crkey, u'places'))
                 if i not in self.intermeds:
-                    self.log.debug('Adding intermediate: '
-                                    + repr(i) + ':' + descr
-                                    + ':' + places)
+                    self.log.debug(u'Adding intermediate: '
+                                    + repr(i) + u':' + descr
+                                    + u':' + places)
                     self.intermeds.append(i)
-                    self.intermap[i] = {'descr':descr, 'places':places}
+                    self.intermap[i] = {u'descr':descr, u'places':places}
                 else:
-                    self.log.info('Ignoring duplicate intermediate: '
+                    self.log.info(u'Ignoring duplicate intermediate: '
                                    + repr(i))
 
         # load contest meta data
         tallyset = set()
-        for i in cr.get('event', 'contests').translate(
-                            strops.BIBLIST_TRANS).split():
+        for i in cr.get(u'event', u'contests'):
             if i not in self.contests:
                 self.contests.append(i)
                 self.contestmap[i] = {}
-                crkey = 'contest_' + i
-                tally = ''
-                if cr.has_option(crkey, 'tally'):
-                    tally = cr.get(crkey, 'tally')
+                crkey = u'contest_' + i
+                tally = u''
+                if cr.has_option(crkey, u'tally'):
+                    tally = cr.get(crkey, u'tally')
                     if tally:
                         tallyset.add(tally)
-                self.contestmap[i]['tally'] = tally
+                self.contestmap[i][u'tally'] = tally
                 descr = i
-                if cr.has_option(crkey, 'descr'):
-                    descr = cr.get(crkey, 'descr')
-                    if descr == '':
+                if cr.has_option(crkey, u'descr'):
+                    descr = cr.get(crkey, u'descr')
+                    if descr == u'':
                         descr = i
-                self.contestmap[i]['descr'] = descr
+                self.contestmap[i][u'descr'] = descr
                 labels = []
-                if cr.has_option(crkey, 'labels'):
-                    labels = cr.get(crkey, 'labels').split()
-                self.contestmap[i]['labels'] = labels
+                if cr.has_option(crkey, u'labels'):
+                    labels = cr.get(crkey, u'labels').split()
+                self.contestmap[i][u'labels'] = labels
                 source = i
-                if cr.has_option(crkey, 'source'):
-                    source = cr.get(crkey, 'source')
-                    if source == '':
+                if cr.has_option(crkey, u'source'):
+                    source = cr.get(crkey, u'source')
+                    if source == u'':
                         source = i
-                self.contestmap[i]['source'] = source
+                self.contestmap[i][u'source'] = source
                 bonuses = []
-                if cr.has_option(crkey, 'bonuses'):
-                    for bstr in cr.get(crkey, 'bonuses').split():
+                if cr.has_option(crkey, u'bonuses'):
+                    for bstr in cr.get(crkey, u'bonuses').split():
                         bt = tod.str2tod(bstr)
                         if bt is None:
-                            self.log.info('Invalid bonus ' + repr(bstr)
-                              + ' in contest ' + repr(i))
+                            self.log.info(u'Invalid bonus ' + repr(bstr)
+                              + u' in contest ' + repr(i))
                             bt = tod.ZERO
                         bonuses.append(bt)
-                self.contestmap[i]['bonuses'] = bonuses
+                self.contestmap[i][u'bonuses'] = bonuses
                 points = []
-                if cr.has_option(crkey, 'points'):
-                    pliststr = cr.get(crkey, 'points').strip()
-                    if pliststr and tally == '': # no tally for these points!
-                        self.log.error('No tally for points in contest: '
+                if cr.has_option(crkey, u'points'):
+                    pliststr = cr.get(crkey, u'points').strip()
+                    if pliststr and tally == u'': # no tally for these points!
+                        self.log.error(u'No tally for points in contest: '
                                         + repr(i))
-                        tallyset.add('')        # add empty placeholder
+                        tallyset.add(u'')       # add empty placeholder
                     for pstr in pliststr.split():
                         pt = 0
                         try:
                             pt = int(pstr)
                         except:
-                            self.log.info('Invalid points ' + repr(pstr)
-                              + ' in contest ' + repr(i))
+                            self.log.info(u'Invalid points ' + repr(pstr)
+                              + u' in contest ' + repr(i))
                         points.append(pt)
-                self.contestmap[i]['points'] = points
+                self.contestmap[i][u'points'] = points
                 allsrc = False          # all riders in source get same pts
-                if cr.has_option(crkey, 'all_source'):
-                    allsrc = strops.confopt_bool(cr.get(crkey, 'all_source'))
-                self.contestmap[i]['all_source'] = allsrc
+                if cr.has_option(crkey, u'all_source'):
+                    allsrc = strops.confopt_bool(cr.get(crkey, u'all_source'))
+                self.contestmap[i][u'all_source'] = allsrc
             else:
-                self.log.info('Ignoring duplicate contest: ' + repr(i))
+                self.log.info(u'Ignoring duplicate contest: ' + repr(i))
 
             # check for invalid allsrc
-            if self.contestmap[i]['all_source']:
-                if (len(self.contestmap[i]['points']) > 1
-                    or len(self.contestmap[i]['bonuses']) > 1):
-                    self.log.info('Ignoring extra points/bonus for '
-                                  + 'all source contest ' + repr(i))
+            if self.contestmap[i][u'all_source']:
+                if (len(self.contestmap[i][u'points']) > 1
+                    or len(self.contestmap[i][u'bonuses']) > 1):
+                    self.log.info(u'Ignoring extra points/bonus for '
+                                  + u'all source contest ' + repr(i))
 
         # load points tally meta data
-        tallylist = cr.get('event', 'tallys').translate(
-                            strops.BIBLIST_TRANS).split()
+        tallylist = cr.get('event', 'tallys')
         # append any 'missing' tallys from points data errors
         for i in tallyset:
             if i not in tallylist:
-                self.log.debug('Adding missing tally to config: ' + repr(i))
+                self.log.debug(u'Adding missing tally to config: ' + repr(i))
                 tallylist.append(i)
         # then scan for meta data
         for i in tallylist:
@@ -492,35 +509,35 @@ class irtt(object):
                 self.tallymap[i] = {}
                 self.points[i] = {}      # redundant, but ok
                 self.pointscb[i] = {}
-                crkey = 'tally_' + i
-                descr = ''
-                if cr.has_option(crkey, 'descr'):
-                    descr = cr.get(crkey, 'descr')
-                self.tallymap[i]['descr'] = descr
+                crkey = u'tally_' + i
+                descr = u''
+                if cr.has_option(crkey, u'descr'):
+                    descr = cr.get(crkey, u'descr')
+                self.tallymap[i][u'descr'] = descr
                 keepdnf = False
-                if cr.has_option(crkey, 'keepdnf'):
-                    keepdnf = strops.confopt_bool(cr.get(crkey, 'keepdnf'))
-                self.tallymap[i]['keepdnf'] = keepdnf
+                if cr.has_option(crkey, u'keepdnf'):
+                    keepdnf = strops.confopt_bool(cr.get(crkey, u'keepdnf'))
+                self.tallymap[i][u'keepdnf'] = keepdnf
             else:
-                self.log.info('Ignoring duplicate points tally: ' + repr(i))
+                self.log.info(u'Ignoring duplicate points tally: ' + repr(i))
 
         # re-join any existing timer state -> no, just do a start
-        self.set_syncstart(tod.str2tod(cr.get('event', 'start')),
-                           tod.str2tod(cr.get('event', 'lstart')))
+        self.set_syncstart(tod.str2tod(cr.get(u'event', u'start')),
+                           tod.str2tod(cr.get(u'event', u'lstart')))
 
         # re-load starters/results
         self.onestart = False
-        for rs in cr.get('event', 'startlist').split():
+        for rs in cr.get(u'event', u'startlist').split():
             (r, s) = strops.bibstr2bibser(rs)
             self.addrider(r, s)
             wst = None
             tst = None
             ft = None
             pt = None
-            if cr.has_option('riders', rs):
+            if cr.has_option(u'riders', rs):
                 # bbb.sss = comment,wall_start,timy_start,finish,penalty,place
                 nr = self.getrider(r, s)
-                ril = csv.reader([cr.get('riders', rs)]).next()
+                ril = cr.get(u'riders', rs)	# vec
                 lr = len(ril)
                 if lr > 0:
                     nr[COL_COMMENT] = ril[0]
@@ -532,131 +549,130 @@ class irtt(object):
                     ft = tod.str2tod(ril[3])
                 if lr > 4:
                     pt = tod.str2tod(ril[4])
-            # check for a start override
-            #if r in stmap:
-                #wst = stmap[r]
             nri = self.getiter(r, s)
             self.settimes(nri, wst, tst, ft, pt, doplaces=False)
 
         self.placexfer()
 
-        self.comment = cr.get('event', 'comment').splitlines()
+        self.comment = cr.get(u'event', u'comment')
 
         # load any 'oncourse' riders from disk -> time does not matter
-        # ERROR: time should be saved and loaded, but for now ok.
-        for r in cr.get('event', 'recentstarts').split():
+        for r in cr.get(u'event', u'recentstarts'):
             self.recent_starts[r]=tod.tod('now')
 
         # if a remote timer configured, create amd start process here
-        self.remport = cr.get('event', 'remote_host')
+        self.remport = cr.get(u'event', u'remote_host')
         if self.remport and not self.readonly:
-            self.remchan = cr.get('event', 'remote_channel')
+            self.remchan = cr.get(u'event', u'remote_channel')
             self.remote_timy = uscbsrv.uscbsrv()
             self.remote_timy.start()
-            self.log.debug('Starting uscbsrv remote timy at: '
+            self.log.debug(u'Starting uscbsrv remote timy at: '
                               + repr(self.remport) + repr(self.remchan))
             self.remote_timy.set_portstr(portstr=self.remport,
                                          channel=self.remchan)
             self.remote_timy.set_pub_cb(self.remote_timy_cb)
         else:
-            self.log.debug('No remote timer configurred.')
+            self.log.debug(u'No remote timer configurred.')
+
+        if strops.confopt_bool(cr.get(u'event', u'finished')):
+            self.set_finished()
 
         # After load complete - check config and report. This ensures
         # an error message is left on top of status stack. This is not
         # always a hard fail and the user should be left to determine
         # an appropriate outcome.
-        eid = cr.get('event', 'id')
+        eid = cr.get(u'event', u'id')
         if eid != EVENT_ID:
-            self.log.error('Event configuration mismatch: '
-                           + repr(eid) + ' != ' + repr(EVENT_ID))
+            self.log.error(u'Event configuration mismatch: '
+                           + repr(eid) + u' != ' + repr(EVENT_ID))
 
     def saveconfig(self):
         """Save race to disk."""
         if self.readonly:
-            self.log.error('Attempt to save readonly ob.')
+            self.log.error(u'Attempt to save readonly ob.')
             return
-        cw = ConfigParser.ConfigParser()
+        cw = jsonconfig.config()
 
         # save basic race properties
-        cw.add_section('event')
-        tstr = ''
+        cw.add_section(u'event')
         if self.start is not None:
-            cw.set('event', 'start', self.start.rawtime())
+            cw.set(u'event', u'start', self.start.rawtime())
         if self.lstart is not None:
-            cw.set('event', 'lstart', self.lstart.rawtime())
-        cw.set('event', 'startlist', self.get_startlist())
-        if self.autocats:
-            cw.set('event', 'categories', 'AUTO')
-        else:
-            cw.set('event', 'categories', self.get_catlist())
-        cw.set('event', 'comment', '\n'.join(self.comment))
+            cw.set(u'event', u'lstart', self.lstart.rawtime())
+        cw.set(u'event', u'comment', self.comment)
         if self.startgap is not None:
-            cw.set('event', 'startgap', self.startgap.rawtime(0))
+            cw.set(u'event', u'startgap', self.startgap.rawtime(0))
 
-        cw.set('event', 'remote_host', self.remport)
-        cw.set('event', 'remote_channel', self.remchan)
+        cw.set(u'event', u'remote_host', self.remport)
+        cw.set(u'event', u'remote_channel', self.remchan)
 
         # save out any oncourse riders
-        stout = ''
+        stout = []
         for r in self.recent_starts:
-            stout += ' ' + r
-        cw.set('event', 'recentstarts', stout)
+            stout.append(r)
+        cw.set(u'event', u'recentstarts', stout)
 
         # save intermediate data
-        cw.set('event', 'intermeds', ' '.join(self.intermeds))
+        cw.set(u'event', u'intermeds', self.intermeds)
         for i in self.intermeds:
-            crkey = 'intermed_' + i
+            crkey = u'intermed_' + i
             cw.add_section(crkey)
-            cw.set(crkey, 'descr', self.intermap[i]['descr'])
-            cw.set(crkey, 'places', self.intermap[i]['places'])
-
+            cw.set(crkey, u'descr', self.intermap[i][u'descr'])
+            cw.set(crkey, u'places', self.intermap[i][u'places'])
         # save contest meta data
-        cw.set('event', 'contests', ' '.join(self.contests))
+        cw.set(u'event', u'contests', self.contests)
         for i in self.contests:
-            crkey = 'contest_' + i
+            crkey = u'contest_' + i
             cw.add_section(crkey)
-            cw.set(crkey, 'tally', self.contestmap[i]['tally'])
-            cw.set(crkey, 'source', self.contestmap[i]['source'])
-            cw.set(crkey, 'all_source', self.contestmap[i]['all_source'])
+            cw.set(crkey, u'tally', self.contestmap[i][u'tally'])
+            cw.set(crkey, u'source', self.contestmap[i][u'source'])
+            cw.set(crkey, u'all_source', self.contestmap[i][u'all_source'])
             blist = []
-            for b in self.contestmap[i]['bonuses']:
+            for b in self.contestmap[i][u'bonuses']:
                 blist.append(b.rawtime(0))
-            cw.set(crkey, 'bonuses', ' '.join(blist))
+            cw.set(crkey, u'bonuses', u' '.join(blist))
             plist = []
-            for p in self.contestmap[i]['points']:
+            for p in self.contestmap[i][u'points']:
                 plist.append(str(p))
-            cw.set(crkey, 'points', ' '.join(plist))
-
+            cw.set(crkey, u'points', u' '.join(plist))
         # save tally meta data
-        cw.set('event', 'tallys', ' '.join(self.tallys))
+        cw.set(u'event', u'tallys', self.tallys)
         for i in self.tallys:
-            crkey = 'tally_' + i
+            crkey = u'tally_' + i
             cw.add_section(crkey)
-            cw.set(crkey, 'descr', self.tallymap[i]['descr'])
-            cw.set(crkey, 'keepdnf', self.tallymap[i]['keepdnf'])
+            cw.set(crkey, u'descr', self.tallymap[i][u'descr'])
+            cw.set(crkey, u'keepdnf', self.tallymap[i][u'keepdnf'])
 
-        # save out all starters
-        cw.add_section('riders')
+        # save riders
+        cw.set(u'event', u'startlist', self.get_startlist())
+        if self.autocats:
+            cw.set(u'event', u'categories', [u'AUTO'])
+        else:
+            cw.set(u'event', u'categories', self.get_catlist())
+        cw.add_section(u'riders')
         for r in self.riders:
             if r[COL_BIB] != '':
+                bib = r[COL_BIB].decode('utf-8')
+                ser = r[COL_SERIES].decode('utf-8')
                 # place is saved for info only
-                wst = ''
+                wst = u''
                 if r[COL_WALLSTART] is not None:
                     wst = r[COL_WALLSTART].rawtime()
-                tst = ''
+                tst = u''
                 if r[COL_TODSTART] is not None:
                     tst = r[COL_TODSTART].rawtime()
-                tft = ''
+                tft = u''
                 if r[COL_TODFINISH] is not None:
                     tft = r[COL_TODFINISH].rawtime()
-                tpt = ''
+                tpt = u''
                 if r[COL_TODPENALTY] is not None:
                     tpt = r[COL_TODPENALTY].rawtime()
-                slice = [r[COL_COMMENT], wst, tst, tft, tpt, r[COL_PLACE]]
-                cw.set('riders', strops.bibser2bibstr(r[COL_BIB],
-                                                      r[COL_SERIES]),
-                    ','.join(map(lambda i: str(i).replace(',', '\\,'), slice)))
-        self.log.debug('Saving race config to: ' + self.configpath)
+                slice = [r[COL_COMMENT].decode('utf-8'),
+                         wst, tst, tft, tpt, r[COL_PLACE]]
+                cw.set(u'riders', strops.bibser2bibstr(bib, ser), slice)
+        cw.set(u'event', u'finished', self.timerstat == 'finished')
+        cw.set(u'event', u'id', EVENT_ID)
+        self.log.debug(u'Saving race config to: ' + repr(self.configpath))
         with open(self.configpath, 'wb') as f:
             cw.write(f)
 
@@ -692,7 +708,6 @@ class irtt(object):
         #self.meet.menu_race_properties.set_sensitive(False)
         if not self.readonly:
             self.saveconfig()
-        self.meet.edb.editevent(self.event, winopen=False)
         self.winopen = False
 
     def do_properties(self):
@@ -734,7 +749,7 @@ class irtt(object):
             if dbr is not None:
                 ucicode = self.meet.rdb.getvalue(dbr,
                                          riderdb.COL_UCICODE)
-            bstr = r[COL_BIB]
+            bstr = r[COL_BIB].decode('utf-8')
             stxt = ''
             if r[COL_WALLSTART] is not None:
                 stxt = r[COL_WALLSTART].meridian()
@@ -743,9 +758,12 @@ class irtt(object):
                         sec.lines.append([None, None, None])	# GAP!!
                 lt = r[COL_WALLSTART]
             nstr = r[COL_NAMESTR]
-            if r[COL_CAT] == 'U23':
-                ucicode += ' *'
-            sec.lines.append([stxt, bstr, nstr, ucicode, '____', None])
+            #if r[COL_CAT] == 'U23':
+                #ucicode += ' *'
+            cat = None
+            if r[COL_CAT]:
+                cat = r[COL_CAT].decode('utf-8')
+            sec.lines.append([stxt, bstr, nstr, ucicode, '____', cat])
         ret.append(sec)
         if cnt > 1:
             sec = printing.section()
@@ -925,6 +943,9 @@ class irtt(object):
                 cat = r[COL_CAT]
                 yield [start, bib, series, name, cat]
 
+    def get_elapsed(self):
+        return None
+
     def result_gen(self, cat=''):
         """Generator function to export a final result."""
         self.placexfer()
@@ -1021,33 +1042,42 @@ class irtt(object):
 
     def rfidtrig(self, e):
         """Register RFID crossing."""
+        if e.refid == '':       # got a trigger
+            self.log.debug(u'Finish trigger via thbc.')
+            return self.fin_trig(e)
+
         r = self.meet.rdb.getrefid(e.refid)
         if r is not None:
             bib = self.meet.rdb.getvalue(r, riderdb.COL_BIB)
             series = self.meet.rdb.getvalue(r, riderdb.COL_SERIES)
             lr = self.getrider(bib, series)
             if lr is not None:
+                # !! TODO: decode bib and series throughout
                 bibstr = strops.bibser2bibstr(lr[COL_BIB], lr[COL_SERIES])
                 #self.meet.scratch_log(' '.join([
                    #bibstr.ljust(5),
                    #strops.truncpad(lr[COL_NAMESTR], 30)
                                               #]))
-                if self.fl.getstatus() in ['idle', 'finish']:
+                flbibstr = strops.bibser2bibstr(self.fl.bibent.get_text(),
+                                                self.fl.serent.get_text())
+                if self.fl.getstatus() not in ['armfin']:
                     if bibstr in self.recent_starts:
                         self.fl.setrider(lr[COL_BIB], lr[COL_SERIES])
                         self.armfinish()
+                        self.log.info(u'Finish armed for: ' + bibstr
+                                      + u'@' + e.rawtime(2))
                         del self.recent_starts[bibstr]
                     else:
-                        self.log.info('Ignoring unseen rider: ' + bibstr
-                                      + '@' + e.rawtime(1))
+                        self.log.info(u'Ignoring rider not on course: '
+                                       + bibstr + u'@' + e.rawtime(2))
                 else:
-                    self.log.info('Finish channel blocked for: '
-                                   + bib + '.' + series + '@' + e.rawtime(1))
+                    self.log.info(u'Finish channel blocked for: '
+                                 + bib + u'.' + series + u'@' + e.rawtime(2))
             else:
-                self.log.info('Non start rider: ' + bib + '.' + series + '@'
-                               + e.rawtime(1))
+                self.log.info(u'Non start rider: ' + bib + u'.' + series
+                                  + u'@' + e.rawtime(2))
         else:
-            self.log.info('Unkown tag: ' + e.refid + '@' + e.rawtime(1))
+            self.log.info(u'Unkown tag: ' + e.refid + u'@' + e.rawtime(1))
         return False
 
 
@@ -1259,7 +1289,7 @@ class irtt(object):
                     lt = t
                 else:
                     self.log.error('Extra result for rider' 
-                                    + strops.bibser2bibstr(t.refid, t.index))
+                                + strops.bibser2bibstr(t.refid, t.index))
 
         # pass two: compute any intermediates
         self.bonuses = {}       # bonuses are global to stage
@@ -1503,8 +1533,10 @@ class irtt(object):
         sel = self.view.get_selection().get_selected()
         if sel is not None:
             self.settimes(sel[1])	# clear iter to empty vals
-            self.log_clear(self.riders.get_value(sel[1], COL_BIB),
-                           self.riders.get_value(sel[1], COL_SERIES))
+            self.log_clear(self.riders.get_value(sel[1],
+                                       COL_BIB).decode('utf-8'),
+                           self.riders.get_value(sel[1],
+                                       COL_SERIES).decode('utf-8'))
 
     def now_button_clicked_cb(self, button, entry=None):
         """Set specified entry to the 'now' time."""
@@ -1601,12 +1633,12 @@ class irtt(object):
         return ret
 
     def get_catlist(self):
-        """Return the ordered list of categories as a string."""
-        ret = []
+        """Return the ordered list of categories."""
+        rvec = []
         for cat in self.cats:
             if cat != '':
-                ret.append(cat)
-        return ','.join(ret)
+                rvec.append(cat)
+        return rvec
 
     def remote_timy_cb(self, msg=None):
         """Handle a remote timer message."""
@@ -1645,18 +1677,18 @@ class irtt(object):
     def __init__(self, meet, event, ui=True):
         """Constructor."""
         self.meet = meet
-        self.event = event      # Note: now a treerowref
-        self.evno = meet.edb.getvalue(event, eventdb.COL_EVNO)
+        self.event = event
+        self.evno = event[u'evid']
         self.configpath = meet.event_configfile(self.evno)
 
         self.log = logging.getLogger('irtt')
         self.log.setLevel(logging.DEBUG)
-        self.log.debug('Creating new irtt event: ' + str(self.evno))
+        self.log.debug(u'opening irtt event: ' + unicode(self.evno))
 
         # properties
         self.remote_timy = None
-        self.remport = ''
-        self.remchan = ''
+        self.remport = u''
+        self.remchan = u''
 
         # race run time attributes
         self.onestart = False
@@ -1669,12 +1701,11 @@ class irtt(object):
         self.startgap = None
         self.cats = []	# the ordered list of cats for results
         self.autocats = False
-        self.results = {'':tod.todlist('UNCAT')}
-        #self.results = tod.todlist('NET')
+        self.results = {u'':tod.todlist(u'UNCAT')}
         self.unstarters = {}
         self.curfintod = None
         self.recent_starts = {}
-        self.curcat = ''
+        self.curcat = u''
         self.comment = []
 
         self.bonuses = {}
@@ -1702,7 +1733,7 @@ class irtt(object):
                                     gobject.TYPE_STRING)   # 10 shortname
 
         b = gtk.Builder()
-        b.add_from_file(os.path.join(metarace.UI_PATH, 'irtt.ui'))
+        b.add_from_file(os.path.join(metarace.UI_PATH, u'irtt.ui'))
 
         self.frame = b.get_object('race_vbox')
         self.frame.connect('destroy', self.shutdown)
@@ -1748,13 +1779,11 @@ class irtt(object):
         if ui:
             b.connect_signals(self)
             b = gtk.Builder()
-            b.add_from_file(os.path.join(metarace.UI_PATH, 'tod_context.ui'))
+            b.add_from_file(os.path.join(metarace.UI_PATH, u'tod_context.ui'))
             self.context_menu = b.get_object('tod_context')
             b.connect_signals(self)
-            self.meet.edb.editevent(event, winopen=True)
             self.meet.alttimer.armlock()   # lock the arm to capture all hits
             self.meet.alttimer.arm(0)
             self.meet.alttimer.arm(1)
             self.meet.timer.setcb(self.rfidtrig)
             self.meet.alttimer.setcb(self.timertrig)
-            self.meet.edb.editevent(event, winopen=True)
