@@ -639,9 +639,9 @@ class omnium(object):
         """Constructor."""
         self.meet = meet
         self.event = event      # Note: now a treerowref
-        self.evno = meet.edb.getvalue(event, eventdb.COL_EVNO)
-        self.evtype = meet.edb.getvalue(event, eventdb.COL_TYPE)
-        self.series = meet.edb.getvalue(event, eventdb.COL_SERIES)
+        self.evno = event[u'evid']
+        self.evtype = event[u'type']
+        self.series = event[u'seri']
         self.configpath = meet.event_configfile(self.evno)
         self.log = logging.getLogger('omnium')
         self.log.setLevel(logging.DEBUG)
