@@ -130,17 +130,21 @@ class rms(object):
         self.cats = []
 
         cr = jsonconfig.config({u'event':{u'start':u'',
-                                        u'lstart':u'',
                                         u'id':EVENT_ID,
                                         u'finish':u'',
                                         u'finished':u'No',
                                         u'places':u'',
                                         u'comment':[],
                                         u'hidecols':[],	# TODO: default hides
-                                        u'categories':u'',
+                                        u'categories':[],
                                         u'intermeds':[],
                                         u'contests':[],
                                         u'tallys':[],
+                                        u'lapstart':u'',
+                                        u'lapfin':u'',
+                                        u'curlap':0,
+                                        u'totlaps':None,
+                                        u'sprintlaps':[],
                                         u'nolaps':u'False',
                                         u'startlist':u'',
                                         u'resultcats':u'False'}})
@@ -195,7 +199,7 @@ class rms(object):
 
         # load contest meta data
         tallyset = set()
-        for i in cr.get('event', 'contests'):
+        for i in cr.get(u'event', u'contests'):
             if i not in self.contests:
                 self.contests.append(i)
                 self.contestmap[i] = {}
@@ -320,9 +324,13 @@ class rms(object):
                 nr[COL_BONUS] = tod.str2tod(cr.get(u'stagebonus', r))
             if cr.has_option(u'stagepenalty', r):
                 nr[COL_PENALTY] = tod.str2tod(cr.get(u'stagepenalty', r))
-        self.set_start(cr.get(u'event', u'start'),
-                       cr.get(u'event', u'lstart'))
+        self.set_start(cr.get(u'event', u'start'))
         self.set_finish(cr.get(u'event', u'finish'))
+        self.lapstart = tod.str2tod(cr.get(u'event', u'lapstart'))
+        self.lapfin = tod.str2tod(cr.get(u'event', u'lapstart'))
+        self.curlap = cr.get(u'event', u'curlap')
+        self.totlaps = cr.get(u'event', u'totlaps')
+        self.sprintlaps = cr.get(u'event', u'sprintlaps')
         self.places = strops.reformat_placelist(cr.get(u'event', u'places'))
         self.comment = cr.get(u'event', u'comment')	# comments are vec
         if strops.confopt_bool(cr.get(u'event', u'finished')):
@@ -422,12 +430,17 @@ class rms(object):
         cw.add_section(u'event')
         if self.start is not None:
             cw.set(u'event', u'start', self.start.rawtime())
-        if self.lstart is not None:
-            cw.set(u'event', u'lstart', self.lstart.rawtime())
         if self.finish is not None:
             cw.set(u'event', u'finish', self.finish.rawtime())
+        if self.lapstart is not None:
+            cw.set(u'event', u'lapstart', self.lapstart.rawtime())
+        if self.lapfin is not None:
+            cw.set(u'event', u'lapfin', self.lapfin.rawtime())
         cw.set(u'event', u'finished', self.timerstat == 'finished')
         cw.set(u'event', u'places', self.places)
+        cw.set(u'event', u'curlap', self.curlap)
+        cw.set(u'event', u'totlaps', self.totlaps)
+        cw.set(u'event', u'sprintlaps', self.sprintlaps)
 
         # save intermediate data
         cw.set(u'event', u'intermeds', self.intermeds)
@@ -1156,7 +1169,7 @@ class rms(object):
                 if r[COL_INRACE]:
                     bt = self.vbunch(r[COL_CBUNCH], r[COL_MBUNCH])
                     ft = bt
-                    if r[COL_STOFT] is not None:
+                    if r[COL_STOFT] is not None and bt is not None:
                         ft = bt - r[COL_STOFT]
 
                 if r[COL_PLACE].isdigit():
@@ -1245,19 +1258,24 @@ class rms(object):
         self.set_start()
         self.clear_results()
         self.timerstat = 'idle'
+        self.meet.announce_cmd(u'timerstat', u'idle')
         uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Idle')
         self.meet.stat_but.set_sensitive(True)
         self.set_elapsed()
+        self.curlap = 0
         self.live_announce = True
         
     def armstart(self):
         """Process an armstart request."""
         if self.timerstat == 'idle':
             self.timerstat = 'armstart'
+            self.meet.announce_cmd(u'timerstat', u'armstart')
+            self.meet.announce_cmd(u'timerstat', u'armstart')
             uiutil.buttonchg(self.meet.stat_but,
                              uiutil.bg_armstart, 'Arm Start')
         elif self.timerstat == 'armstart':
             self.timerstat = 'idle'
+            self.meet.announce_cmd(u'timerstat', u'idle')
             uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Idle') 
         elif self.timerstat == 'ready':
             self.set_running()
@@ -1265,12 +1283,17 @@ class rms(object):
     def armfinish(self):
         """Process an armfinish request."""
         if self.timerstat in ['ready', 'running', 'finished']:
+            if self.totlaps and self.curlap < self.totlaps:
+                # assume lap was not armed
+                self.armlap()
             self.timerstat = 'armfinish'
+            self.meet.announce_cmd(u'timerstat', u'armfinish')
             uiutil.buttonchg(self.meet.stat_but,
                              uiutil.bg_armfin, 'Arm Finish')
             self.meet.stat_but.set_sensitive(True)
         elif self.timerstat == 'armfinish':
             self.timerstat = 'running'
+            self.meet.announce_cmd(u'timerstat', u'running')
             uiutil.buttonchg(self.meet.stat_but,
                              uiutil.bg_none, 'Running')
 
@@ -1282,6 +1305,48 @@ class rms(object):
                 ret = r[COL_BIB]
         return ret
         
+    def armlap(self):
+        ## announce text handling...
+        self.scratch_tot = 0
+        self.scratch_map = {}
+        self.scratch_ord = []
+        if self.live_announce:
+            self.meet.announce_clear()
+            self.meet.announce_title(self.title_namestr.get_text())
+        if self.timerstat in ['idle', 'armstart']:
+            self.meet.announce_cmd(u'finstr', self.meet.get_short_name())
+            return True	# no arm till event underway
+
+        if self.curlap == 0 or self.lapfin is not None:
+            self.curlap += 1	# increment
+            if self.totlaps and self.curlap > self.totlaps:
+                self.log.info('Too many laps.')
+                self.curlap = self.totlaps
+
+        # Write lap time fields
+        if self.timerstat not in ['armfinish', 'finished']:
+            self.meet.announce_cmd(u'finstr', None)
+            ## Step 1: lap time handling
+            if self.lapfin:
+                # roll over to lap start
+                self.lapstart = self.lapfin
+            elif self.lapstart:# assume still waiting for same lap
+                pass
+            else:		# at start?
+                self.lapstart = self.start
+            lapstr = None
+            if self.totlaps is not None:
+                lapstr = (u'Lap ' + unicode(self.curlap)
+                                  + u'/' + unicode(self.totlaps))
+                self.lapentry.set_text(lapstr)
+            self.meet.announce_cmd(u'laplbl', lapstr)
+            if self.curlap in self.sprintlaps:
+                self.meet.announce_cmd(u'laptype',u'SPRINT')
+            else:
+                self.meet.announce_cmd(u'laptype',None)
+            self.lapfin = None
+        self.reannounce_times()
+
     def key_event(self, widget, event):
         """Handle global key presses in event."""
         if event.type == gtk.gdk.KEY_PRESS:
@@ -1290,6 +1355,9 @@ class rms(object):
                 if key == key_abort:    # override ctrl+f5
                     self.resettimer()
                     return True
+                elif key == key_announce: # re-send current announce vars
+                    self.reannounce_times()
+                    return True
                 elif key == key_confirm:  # do recalc with current model
                     lbib = self.last_rftime()
                     if lbib is not None:
@@ -1315,20 +1383,8 @@ class rms(object):
                     self.set_finished()
                     return True
                 elif key == key_armlap:
-                    #self.meet.scratch_clear()
-                    if self.live_announce:
-                        self.meet.announce_clear()
-                        #self.meet.announce_title(self.title_namestr.get_text())
-                        lapstr = self.lapentry.get_text()
-                        if not lapstr:
-                            lapstr = self.title_namestr.get_text()
-                        self.meet.announce_title(lapstr)
-                        self.meet.announce_gap()
-                    self.last_scratch = self.scratch_start
-                    self.scratch_start = None
-                    self.scratch_tot = 0
-                    self.scratch_map = {}
-                    self.scratch_ord = []
+                    ## lap timing vars
+                    self.armlap()
                     return True
                 elif key == key_markrider:
                     self.set_ridermark()
@@ -1427,11 +1483,20 @@ class rms(object):
     def starttrig(self, e):
         """Process a 'start' trigger signal."""
         if self.timerstat == 'armstart':
-            self.set_start(e, tod.tod('now'))
+            self.set_start(e)
         return False
 
+    def set_lap_finish(self, e):
+        """Write lap time into model and emit changed state."""
+        self.lapfin = e
+        self.meet.announce_cmd(u'laptype',None)
+        self.reannounce_times()
+
     def rfidtrig(self, e):
         """Process rfid event."""
+        # bounce message onto announcer: HACK
+        self.meet.announce_timer(e)
+
         if e.refid == '':	# got a trigger
             return self.starttrig(e)
 
@@ -1479,8 +1544,15 @@ class rms(object):
                 lr[COL_RFTIME] = e
                 self.__dorecalc = True	# cleared in timeout, from same thread
                 if lr[COL_INRACE]:
-                    if self.scratch_start is None:
-                        self.scratch_start = e
+                    if self.lapfin is None:
+                        if lr[COL_LAPS] == self.curlap:
+                            self.set_lap_finish(e)
+                        else:
+                            self.log.info(u'Lap finish ignored: ' + 
+                                       lr[COL_BIB] + u'@' + e.rawtime(3)
+                                       + u' on lap ' + unicode(lr[COL_LAPS]))
+                    #if self.scratch_start is None:
+                        #self.scratch_start = e
                     self.announce_rider('', bib, lr[COL_NAMESTR],
                                     lr[COL_CAT], e)
             else:
@@ -1489,8 +1561,15 @@ class rms(object):
         elif self.timerstat in 'running':
             lr[COL_LAPS] += 1
             if lr[COL_INRACE]:
-                if self.scratch_start is None:
-                    self.scratch_start = e
+                if self.lapfin is None:
+                    if lr[COL_LAPS] == self.curlap:
+                        self.set_lap_finish(e)
+                    else:
+                        self.log.info(u'Lap finish ignored: ' + 
+                                   lr[COL_BIB] + u'@' + e.rawtime(3)
+                                  + u' on lap ' + unicode(lr[COL_LAPS]))
+                #if self.scratch_start is None:
+                    #self.scratch_start = e
                 self.announce_rider('', bib, lr[COL_NAMESTR],
                                 lr[COL_CAT], e)
         return False	# called from glib_idle_add
@@ -1509,16 +1588,15 @@ class rms(object):
 
         self.live_announce = False
         self.meet.announce_clear()
-        ## needs better annotation?
-        #self.meet.announce_title(self.title_namestr.get_text()
+        scrollmsg = u'Provisional - '
         self.meet.announce_title('Provisional Result')
+        self.meet.announce_cmd(u'bunches',u'final')
         placeset = set()
         idx = 0
         st = tod.tod('0')
         if self.start is not None:
             st = self.start
-        # result is sent in final units, not absolutes
-        self.meet.announce_start()
+        # result is sent in weird hybrid units TODO: fix the amb
         wt = None
         lb = None
         for placegroup in places.split():
@@ -1532,27 +1610,28 @@ class rms(object):
                                          r[COL_MBUNCH])
                         fs = ''
                         if ft is not None:
-                            #if ft != lb:
-                                #fs = ft.rawtime()
-                            #else:
-                                #if r[COL_RFTIME] is not None:
-                                    #fs = (r[COL_RFTIME]-st).rawtime()
-                                #else:
-                                    #fs = ft.rawtime()
                             # temp -> just use the no-blob style to correct
                             fs = ft.rawtime()
                             if wt is None:
                                 wt = ft
                             lb = ft
+                        scrollmsg += (u' ' +
+                                     r[COL_PLACE] + u'. ' 
+                                     + r[COL_NAMESTR].decode('utf-8')
+                                     + u' ')
                         glib.idle_add(self.meet.announce_rider,
                                                  [r[COL_PLACE]+'.',
                                                  bib,
                                                  r[COL_NAMESTR],
                                                  r[COL_CAT], fs])
                     idx += 1
+        self.meet.announce_cmd(u'scrollmsg',scrollmsg)
         if wt is not None:
+            self.meet.announce_cmd(u'start',self.start.rawtime())
+            self.meet.announce_cmd(u'finish',self.finish.rawtime())
             # set winner's time
-            self.meet.announce_time(wt.rawtime(0))
+            ## TODO: is this valid?
+            #self.meet.announce_time(wt.rawtime(0))
 
     def intsprint(self, acode='', places=''):
         """Display an intermediate sprint 'provisional' result."""
@@ -1567,7 +1646,9 @@ class rms(object):
             descr = self.intermap[acode]['descr']
         self.live_announce = False
         self.meet.announce_clear()
+        self.reannounce_times()
         self.meet.announce_title(descr)
+        scrollmsg = descr + u' - '
         placeset = set()
         idx = 0
         for placegroup in places.split():
@@ -1577,6 +1658,10 @@ class rms(object):
                     placeset.add(bib)
                     r = self.getrider(bib)
                     if r is not None:
+                        scrollmsg += (u' ' +
+                                     str(curplace) + u'. ' 
+                                     + r[COL_NAMESTR].decode('utf-8')
+                                     + u' ')
                         glib.idle_add(self.meet.announce_rider,
                                                 [str(curplace)+'.',
                                                  bib,
@@ -1585,18 +1670,34 @@ class rms(object):
                     idx += 1
                 else:
                     self.log.warn('Duplicate no. = ' + str(bib) + ' in places.')
-
+        self.meet.announce_cmd(u'scrollmsg',scrollmsg)
         glib.timeout_add_seconds(15, self.reannounce_lap)
 
+    def todempty(self, val):
+        if val is not None:
+            return val.rawtime()
+        else:
+            return u''
+
+    def reannounce_times(self):
+        """Re-send the current timing values."""
+        self.meet.announce_cmd(u'timerstat', self.timerstat)
+        self.meet.announce_cmd(u'start', self.todempty(self.start))
+        self.meet.announce_cmd(u'finish', self.todempty(self.finish))
+        self.meet.announce_cmd(u'lapstart', self.todempty(self.lapstart))
+        self.meet.announce_cmd(u'lapfin', self.todempty(self.lapfin))
+        return False
+
     def reannounce_lap(self):
-        self.live_announce = False
+        """Re-send current lap passing data."""
         self.meet.announce_clear()
+        self.meet.announce_cmd(u'scrollmsg', None)
+        self.reannounce_times()
+        self.live_announce = False
         if self.timerstat == 'armfinish':
             self.meet.announce_title('Finish')
         else:
             self.meet.announce_title(self.title_namestr.get_text())
-        if self.last_scratch is not None:
-            self.meet.announce_start(self.last_scratch)
         for bib in self.scratch_ord:
             r = self.getrider(bib)
             if r is not None:
@@ -1618,20 +1719,16 @@ class rms(object):
             self.recalculate()
         return True
 
-    def set_start(self, start='', lstart=''):
+    def set_start(self, start=''):
         """Set the start time."""
+        self.lapstart = None
+        self.lapfin = None
         if type(start) is tod.tod:
             self.start = start
         else:
             self.start = tod.str2tod(start)
-        if type(lstart) is tod.tod:
-            self.lstart = lstart
-        else:
-            self.lstart = tod.str2tod(lstart)
-            if self.lstart is None:
-                self.lstart = self.start
         if self.start is not None:
-            self.meet.announce_start(self.start)
+            self.meet.announce_cmd(u'start', self.start.rawtime())
             self.last_scratch = self.start
             if self.finish is None:
                 self.set_ready()
@@ -1646,14 +1743,15 @@ class rms(object):
             if self.start is not None:
                 self.set_ready()
         else:
+            self.meet.announce_cmd(u'finish', self.finish.rawtime())
             if self.start is None:
                 self.set_start('0')
 
     def get_elapsed(self):
         """Hack mode - always returns time from start."""
         ret = None
-        if self.lstart is not None and self.timerstat != 'finished':
-            ret = (tod.tod('now') - self.lstart).truncate(0)
+        if self.start is not None and self.timerstat != 'finished':
+            ret = (tod.tod('now') - self.start).truncate(0)
         return ret
 
     def set_elapsed(self):
@@ -1661,40 +1759,31 @@ class rms(object):
         if self.start is not None and self.finish is not None:
             elap = (self.finish - self.start).truncate(0)
             self.time_lbl.set_text(elap.rawtime(0))
-            # finish gap time
-            if self.timerstat != 'finished':
-                self.meet.announce_gap('+' + (tod.tod('now') - self.finish.truncate(0)).rawtime(0))
-                if self.meet.distance is not None:
-                    self.meet.announce_avg(elap.speedstr(1000.0*self.meet.distance))
-            else:
-                self.meet.announce_gap()
-        elif self.start is not None:    # Note: uses 'local start' for RT
-            self.time_lbl.set_text((tod.tod('now') - self.lstart).rawtime(0))
-            # lap gap time
-            if self.scratch_start is not None:
-                self.meet.announce_gap('+' + (tod.tod('now') - self.scratch_start.truncate(0)).rawtime(0))
+        elif self.start is not None:
+            self.time_lbl.set_text((tod.tod('now') - self.start).rawtime(0))
         elif self.timerstat == 'armstart':
             self.time_lbl.set_text(tod.tod(0).rawtime(0))
-            self.meet.announce_gap()
         else:
             self.time_lbl.set_text('')
-            self.meet.announce_gap()
-
-        self.meet.announce_time(self.time_lbl.get_text())
 
     def set_ready(self):
         """Update event status to ready to go."""
         self.timerstat = 'ready'
+        self.meet.announce_cmd(u'timerstat', u'ready')
         uiutil.buttonchg(self.meet.stat_but, uiutil.bg_armint, 'Ready')
 
     def set_running(self):
         """Update event status to running."""
         self.timerstat = 'running'
+        self.meet.announce_cmd(u'timerstat', u'running')
         uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Running')
 
     def set_finished(self):
         """Update event status to finished."""
         self.timerstat = 'finished'
+        self.meet.announce_cmd(u'timerstat', u'finished')
+        self.meet.announce_cmd(u'laplbl', None)
+        self.meet.announce_cmd(u'laptype', None)
         uiutil.buttonchg(self.meet.stat_but, uiutil.bg_none, 'Finished')
         self.meet.stat_but.set_sensitive(False)
         if self.finish is None:
@@ -1713,7 +1802,7 @@ class rms(object):
                     nplaces += ' ' + r[COL_BIB]
                     lplace = r[COL_PLACE]
         self.places = strops.reformat_placelist(nplaces)
-        self.meet.action_combo.set_active(5)
+        self.meet.action_combo.set_active(10)	# 'fin'
         self.meet.action_entry.set_text(self.places)
         
     def fill_places_to(self, bib=None):
@@ -2137,12 +2226,10 @@ class rms(object):
     def new_start_trigger(self, rfid):
         """Collect a RFID trigger signal and apply it to the model."""
         if self.newstartdlg is not None and self.newstartent is not None:
-            nt = tod.tod('now')
             et = tod.str2tod(self.newstartent.get_text())
             if et is not None:
                 st = rfid - et
-                lt = nt - et
-                self.set_start(st, lt)
+                self.set_start(st)
                 self.newstartdlg.response(1)
                 self.newstartdlg = None	# try to ignore the 'up' impulse
             else:
@@ -2242,14 +2329,14 @@ class rms(object):
 
     def __init__(self, meet, event, ui=True):
         self.meet = meet
-        self.event = event      # Note: now a treerowref
+        self.event = event
         self.evno = event[u'evid']
         self.series = event[u'seri']
         self.configpath = meet.event_configfile(self.evno)
 
         self.log = logging.getLogger('roadrace')
         self.log.setLevel(logging.DEBUG)
-        self.log.debug('opening event: ' + str(self.evno))
+        self.log.debug(u'opening event: ' + unicode(self.evno))
 
         self.recalclock = threading.Lock()
         self.__dorecalc = False
@@ -2259,7 +2346,6 @@ class rms(object):
         # race run time attributes
         self.readonly = not ui
         self.start = None
-        self.lstart = None
         self.finish = None
         self.winopen = True
         self.timerstat = 'idle'
@@ -2286,6 +2372,11 @@ class rms(object):
         self.scratch_last = None
         self.scratch_count = 0
         self.scratch_tot = 0
+        self.curlap = 0
+        self.sprintlaps = []
+        self.totlaps = None
+        self.lapstart = None
+        self.lapfin = None
  
         # lap tracking
         self.scratch_map = {}
@@ -2336,6 +2427,7 @@ class rms(object):
         self.frame.connect('destroy', self.shutdown)
 
         # meta info pane
+        self.shortname = None
         self.title_namestr = b.get_object('title_namestr')
         self.set_titlestr()
         self.time_lbl = b.get_object('time_lbl')
