@@ -203,7 +203,6 @@ class registerdlg(object):
 class fakemeet:
     """Road meet placeholder for external event manipulations."""
     def __init__(self, edb, rdb, path):
-        raise NotImplementedError(u'NOT IMPLEMENTED.')
         self.edb = edb
         self.rdb = rdb
         self.configpath = path
@@ -219,6 +218,11 @@ class fakemeet:
         self.distance = None
         self.docindex = 0
         self.bibs_in_results = True
+    def get_distance(self):
+        return self.distance
+
+    def announce_cmd(self, ctype=None, cmsg=None):
+        pass
     def announce_time(self, data=None):
         pass
     def announce_clear(self):
@@ -233,40 +237,43 @@ class fakemeet:
         pass
     def loadconfig(self):
         """Load meet config from disk."""
-        raise NotImplementedError(u'NOT IMPLEMENTED.')
-        cr = ConfigParser.ConfigParser({
-               'title':'',
-               'date':'',
-               'organiser':'',
-               'commissaire':'',
-               'logo':'',
-               'distance':'',
-               'docindex':'0',
-               'resultnos':'Yes',
-               'id':''
-        })
-        cr.add_section('meet')
+        cr = jsonconfig.config({u'meet':{
+               u'title':u'',
+               u'shortname':u'',
+               u'subtitle':u'',
+               u'date':u'',
+               u'organiser':u'',
+               u'commissaire':u'',
+               u'logo':u'',
+               u'distance':u'',
+               u'docindex':u'0',
+               u'resultnos':u'Yes',
+               u'uscbport':u'',
+               u'id':u''}})
+        cr.add_section(u'meet')
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
         # check for config file
         try:
-            a = len(cr.read(cwfilename))
-            if a == 0:
-                self.log.warn('No config file - loading default values.')
-        except e:
-            self.log.error('Error reading meet config: ' + str(e))
+            with open(cwfilename, 'rb') as f:
+                cr.read(f)
+        except Exception as e:
+            self.log.error(u'Reading meet config: ' + unicode(e))
         # set meet meta, and then copy into text entries
-        self.title_str = cr.get('meet', 'title')
-        self.subtitle_str = cr.get('meet', 'subtitle')
-        self.date_str = cr.get('meet', 'date')
-        self.organiser_str = cr.get('meet', 'organiser')
-        self.commissaire_str = cr.get('meet', 'commissaire')
-        self.logo = cr.get('meet', 'logo')
-        self.distance = strops.confopt_float(cr.get('meet', 'distance'))
-        self.docindex = strops.confopt_posint(cr.get('meet', 'docindex'), 0)
-        self.bibs_in_results = strops.confopt_bool(cr.get('meet', 'resultnos'))
+        self.shortname = cr.get(u'meet', u'shortname')
+        self.title_str = cr.get(u'meet', u'title')
+        self.subtitle_str = cr.get(u'meet', u'subtitle')
+        self.date_str = cr.get(u'meet', u'date')
+        self.organiser_str = cr.get(u'meet', u'organiser')
+        self.commissaire_str = cr.get(u'meet', u'commissaire')
+        self.logo = cr.get(u'meet', u'logo')
+        self.distance = strops.confopt_float(cr.get(u'meet', u'distance'))
+        self.docindex = strops.confopt_posint(cr.get(u'meet', u'docindex'), 0)
+        self.bibs_in_results = strops.confopt_bool(cr.get(u'meet',
+                                                          u'resultnos'))
     def event_configfile(self, evno):
         """Return a config filename for the given event no."""
-        return os.path.join(self.configpath, 'event_' + str(evno) + '.json')
+        return os.path.join(self.configpath, u'event_' 
+                                     + unicode(evno) + u'.json')
 
 class roadmeet:
     """Road meet application class."""
@@ -296,6 +303,10 @@ class roadmeet:
         self.saveconfig()
         self.log.info(u'Meet data saved to ' + repr(self.configpath) + u'.')
 
+    def get_short_name(self):
+        """Return the <= 16 char shortname."""
+        return self.shortname
+
     ## TODO: call into curevent and fetch configurables + use generic props
     def menu_meet_properties_cb(self, menuitem, data=None):
         """Edit meet properties."""
@@ -330,6 +341,8 @@ class roadmeet:
         upb = b.get_object('scb_port_dfl')
         upb.connect('clicked', 
                     lambda b: upe.set_text('uSCBsrv@localhost'))
+        uoc = b.get_object('uscbsrv_option_check')
+        uoc.set_active(self.remote_enable)
         mte = b.get_object('timing_main_entry')
         mte.set_text(self.timer_port)
         mtb = b.get_object('timing_main_dfl')
@@ -352,6 +365,7 @@ class roadmeet:
             self.distance = strops.confopt_float(
                                 di_ent.get_text().decode('utf-8'))
             self.bibs_in_results = n_opt.get_active()
+            self.remote_enable = uoc.get_active()
             self.set_title()
 
             nchan = uce.get_text().decode('utf-8')
@@ -361,6 +375,7 @@ class roadmeet:
                 self.uscbchan = nchan
                 self.scb.set_portstr(portstr=self.uscbport,
                                      channel=self.uscbchan)
+            self.remote_reset()
             self.set_timer(mte.get_text())
             nport = ate.get_text()
             if nport != self.alttimer_port:
@@ -685,13 +700,14 @@ class roadmeet:
                 cw = ucsv.UnicodeWriter(f)
                 cw.writerow([u'Rank', u'No.', u'Time', u'Bonus', u'Penalty'])
                 for r in self.curevent.result_gen(u''):
-                    if r[2] is not None:
-                        r[2] = unicode(r[2].timeval)	# take raw value out
-                    if r[3] is not None:
-                        r[3] = r[3].rawtime(0)
-                    if r[4] is not None:
-                        r[4] = r[4].rawtime(0)
-                    cw.writerow(r)
+                    opr = [u'',u'',u'',u'',u'']
+                    for i in range(0,2):
+                        if r[i]:
+                            opr[i] = unicode(r[i])
+                    for i in range(2,5):
+                        if r[i]:
+                            opr[i] = unicode(r[i].timeval)
+                    cw.writerow(opr)
             self.log.info(u'Exported meet results to ' + repr(rfilename))
 
     def menu_export_startlist_activate_cb(self, menuitem, data=None):
@@ -764,7 +780,7 @@ class roadmeet:
         filebase = os.path.basename(self.configpath)
         if filebase in [u'', u'.']:
             filebase = u'result'
-            self.log.warn(u'Using default flename for export: result')
+            self.log.warn(u'Using default filename for export: result')
         else:
             pass
             #filebase += u'_result'
@@ -1003,10 +1019,12 @@ class roadmeet:
         cw.set(u'meet', u'id', ROADMEET_ID)
         cw.set(u'meet', u'uscbport', self.uscbport)
         cw.set(u'meet', u'uscbchan', self.uscbchan)
+        cw.set(u'meet', u'uscbopt', self.remote_enable)
         cw.set(u'meet', u'timer', self.timer_port)
         cw.set(u'meet', u'rfidlvl', self.rfidlvl)
         cw.set(u'meet', u'alttimer', self.alttimer_port)
 
+        cw.set(u'meet', u'shortname', self.shortname)
         cw.set(u'meet', u'title', self.title_str)
         cw.set(u'meet', u'subtitle', self.subtitle_str)
         cw.set(u'meet', u'date', self.date_str)
@@ -1055,6 +1073,7 @@ class roadmeet:
     def loadconfig(self):
         """Load meet config from disk."""
         cr = jsonconfig.config({u'meet':{
+               u'shortname':None,
                u'title':u'',
                u'subtitle':u'',
                u'date':u'',
@@ -1096,11 +1115,20 @@ class roadmeet:
         except Exception as e:
             self.log.error(u'Reading meet config: ' + unicode(e))
 
+        # configurable log level on UI (does not change log file)
+        self.loglevel = strops.confopt_posint(cr.get(u'meet', u'loglevel'),
+                                              logging.INFO)
+        self.lh.setLevel(self.loglevel)
+        self.log.debug(u'UI Log level: ' + repr(self.loglevel))
+        self.log.debug(u'Reading event config.')
+
         # set uSCBsrv connection
         self.uscbchan = cr.get(u'meet', u'uscbchan')
         self.uscbport = cr.get(u'meet', u'uscbport')
         self.scb.set_portstr(portstr=self.uscbport,
                              channel=self.uscbchan)
+        self.remote_enable = strops.confopt_bool(cr.get(u'meet', u'uscbopt'))
+        self.remote_reset()
 
         # set timer port (decoder)
         self.set_timer(cr.get(u'meet', u'timer'))
@@ -1121,6 +1149,7 @@ class roadmeet:
             self.alttimer.sane() # sane prod here is probably good idea
 
         # set meet meta, and then copy into text entries
+        self.shortname = cr.get(u'meet', u'shortname')
         self.title_str = cr.get(u'meet', u'title')
         self.subtitle_str = cr.get(u'meet', u'subtitle')
         self.date_str = cr.get(u'meet', u'date')
@@ -1134,11 +1163,6 @@ class roadmeet:
         self.mirrorpath = cr.get(u'meet', u'mirrorpath')
         self.mirrorcmd = cr.get(u'meet', u'mirrorcmd')
 
-        # configurable log level on UI (does not change log file)
-        self.loglevel = strops.confopt_posint(cr.get(u'meet', u'loglevel'),
-                                              logging.INFO)
-        self.lh.setLevel(self.loglevel)
-
         # Re-Initialise rider and event databases
         self.rdb.clear()
         self.edb.clear()
@@ -1160,7 +1184,7 @@ class roadmeet:
             try:
                 self.printprefs.load_file(psfilename)
             except Exception as e:
-                self.log.warn(u'Error loading print preferences: '
+                self.log.debug(u'Error loading print preferences: '
                                  + unicode(e))
 
         # After load complete - check config and report. This ensures
@@ -1171,11 +1195,19 @@ class roadmeet:
         if cid and cid != ROADMEET_ID:
             self.log.error(u'Meet configuration mismatch: '
                            + repr(cid) + u' != ' + repr(ROADMEET_ID))
+        self.log.debug(u'Event config done.')
 
     def get_distance(self):
         """Return race distance in km."""
         return self.distance
 
+    def announce_timer(self, evt):
+        """Send message into announce for remote control."""
+        if not self.remote_enable:
+            tvec = [unicode(1), self.timer.unitno, evt.chan,
+                    evt.refid, evt.rawtime()]
+            self.scb.send_cmd(u'timer',unichr(unt4.US).join(tvec))
+
     def announce_clear(self):
         """Clear announce panels."""
         self.announce_model.clear()
@@ -1192,20 +1224,9 @@ class roadmeet:
             self.an_cur_start = starttod
         self.scb.set_start(starttod)
 
-    def announce_time(self, timestr=None):
-        """Set the announcer time."""
-        if timestr is None:
-            timestr = tod.tod('now').rawtime(0)
-        # no local announce time?
-        self.scb.set_time(timestr)
-
-    def announce_gap(self, timestr=''):
-        """Set the announcer gap time."""
-        self.scb.set_gap(timestr)
-
-    def announce_avg(self, timestr=''):
-        """Set the announcer avg."""
-        self.scb.set_avg(timestr)
+    def announce_cmd(self, ctype=None, cmsg=None):
+        """Announce an generic uscbsrv command."""
+        self.scb.send_cmd(ctype, cmsg)
 
     def announce_rider(self, rvec):
         """Announce the supplied rider vector."""
@@ -1270,6 +1291,42 @@ class roadmeet:
             self.mirror.start()
         return False    # for idle_add
 
+    def remote_reset(self):
+        """Reset remote control callbacks and status."""
+        # TODO: Reset authentication variables
+        # TODO: Private message callbacks
+        self.log.debug(u'Remote control reset.')
+        if self.remote_enable:
+            self.log.debug(u'Enabled with:' + repr(self.remote_pub_cb))
+            self.scb.set_pub_cb(self.remote_pub_cb)
+        else:
+            self.log.debug(u'Disabled.')
+            self.scb.set_pub_cb()
+
+    def remote_pub_cb(self, cmd, nick):
+        """Handle a remote control message."""
+        # TODO : are we listening for this nick?
+        # TODO : is the sender authenticated to control?
+        # TODO : is this a message already dealt with?
+        # TODO : is it possible some data was missed?
+        if cmd.header == u'timer':
+            tv = cmd.text.split(unichr(unt4.US)) # split into fields
+            if len(tv) > 4:	# enough fields
+                try:
+                    index = int(tv[0])	# record index
+                    ## TODO: handle discontinuity
+                    srcid = tv[1]
+                    self.log.debug(u'Remote Timer: ' + repr(srcid)
+                                   + u' index:' + repr(index) + u' '
+                                   + tv[2] + u'/' + tv[3] + u'@' + tv[4])
+                    self.timer.trig(timeval=tv[4], index='REM',
+                                    chan=tv[2], refid=tv[3])
+                except Exception as e:
+                    self.log.error('Reading timer msg: ' + repr(e))
+        else:
+            self.log.debug(u'Unknown remote cmd ' + repr(nick) + u': '
+                       + repr(cmd.header) + u'::' + repr(cmd.text))
+
     def __init__(self, configpath=None, etype=None):
         """Meet constructor."""
         # logger and log handler
@@ -1285,6 +1342,7 @@ class roadmeet:
         if configpath is None:
             configpath = u'.'	# None assumes 'current dir'
         self.configpath = configpath
+        self.shortname = None
         self.title_str = u''
         self.subtitle_str = u''
         self.date_str = u''
@@ -1294,6 +1352,7 @@ class roadmeet:
         self.distance = None
         self.docindex = 0
         self.bibs_in_results = True
+        self.remote_enable = False
         self.loglevel = logging.INFO	# UI log window
 
         # printer preferences
