@@ -55,14 +55,17 @@ THBCPHOTOTHRESH = tod.tod('0.02')
 # THbC protocol messages
 ESCAPE = chr(0x1b)
 HELOCMD = 'MR1'
-STARTCMD = ESCAPE + chr(0x07)
 STOPCMD = ESCAPE + chr(0x13) + chr(0x5c)
 REPEATCMD = ESCAPE + chr(0x12)
 ACKCMD = ESCAPE + chr(0x11)
-STATCMD = ESCAPE + chr(0x05)
-QUECMD = ESCAPE + chr(0x10)	# fetch configuration
+
+STATCMD = ESCAPE + chr(0x05)	# fetch status
+CHKCMD = ESCAPE + chr(0x06)	# UNKNOWN
+STARTCMD = ESCAPE + chr(0x07)	# start decoder
 SETCMD = ESCAPE + chr(0x08)	# set configuration
 IPCMD = ESCAPE + chr(0x09)	# set IP configuration
+QUECMD = ESCAPE + chr(0x10)	# fetch configuration
+
 STALVL = ESCAPE + chr(0x1e)
 BOXLVL = ESCAPE + chr(0x1f)
 
@@ -100,6 +103,8 @@ class thbc(threading.Thread):
         self.port = None
         self.error = False
         self.errstr = ''
+        self.unitno = None		# fetched on reload
+        self.decoderflags = []		# fetched on reload
         self.cqueue = Queue.Queue()	# command queue
         self.log = logging.getLogger(self.name)
         self.log.setLevel(logging.DEBUG)
@@ -184,6 +189,10 @@ class thbc(threading.Thread):
         """Request status message from decoder."""
         self.write(STATCMD)
 
+    def get_config(self):
+        """Request decoder configuration."""
+        self.write(QUECMD)
+
     def setlvl(self, box=u'10', sta=u'10'):
         """Set the read level on box and sta channels."""
         # TODO: verify opts
@@ -262,6 +271,25 @@ class thbc(threading.Thread):
                     self.log.info(msg.strip())
                 else:
                     self.log.info('Invalid status: ' + repr(msg))
+            elif '+++' == msg[0:3] and len(msg) > 53:
+                # decoder configuration message.
+                self.decoderflags = []
+                flags = { 3:u'Time of Day',
+                          4:u'GPS Enable',
+                         10:u'Detect Max'}
+                for bf in [3, 4, 10]:
+                    if ord(msg[bf]):	# Time of day
+                        self.decoderflags.append(flags[bf])
+                self.unitno = u''
+                for c in msg[46:50]:
+                    self.unitno += unichr(ord(c)+ord('0'))
+                stalvl = hex(ord(msg[28]))	# ? question this
+                boxlvl = hex(ord(msg[29]))
+
+                self.log.info(u'Decoder ID: ' + self.unitno 
+                               + '\tFlags: ' + u','.join(self.decoderflags))
+                self.log.debug(u'Decoder Levels: STA=' + stalvl
+                               + u', BOX=' + boxlvl)
             else:
                 self.log.log(RFID_LOG_LEVEL, repr(msg))
         else:        
@@ -278,7 +306,7 @@ class thbc(threading.Thread):
             elif ch == LF:
                 # Newline ends the current 'message'
                 self.__rdbuf += ch	# include trailing newline
-                t = self.__parse_message(self.__rdbuf)
+                t = self.__parse_message(self.__rdbuf.lstrip('\0'))
                 if t is not None:
                     glib.idle_add(self.__cb, t)
                 self.__rdbuf = ''
@@ -309,7 +337,7 @@ class thbc(threading.Thread):
                 
                 # Write phase
                 if type(m) is tuple and type(m[0]) is str and m[0] in TCMDS:
-                    if m[0] == 'MSG' and not self.error:
+                    if m[0] == 'MSG' and self.port and not self.error:
                         cmd = m[1] ##+ '\r\n'
                         self.log.debug('Sending rawmsg ' + repr(cmd))
                         self.port.write(cmd)
@@ -339,6 +367,8 @@ class thbc(threading.Thread):
                             self.port = serial.Serial(m[1], THBC_BAUD,
                                                   rtscts=False, timeout=0.2)
                             self.error = False
+                            self.unitno = None
+                            self.get_config()	# re-identify decoder
                         else:
                             self.log.debug('Not connected.')
                             self.error = True
