@@ -78,19 +78,18 @@ import serial
 import logging
 import decimal
 import glib
-import metarace
+import metarace	# already in this module?
 
-from metarace import strops
 from metarace import tod
 
 # System default timy serial port
-DEFPORT = '/dev/ttyUSB0'
-TIMYPORT = '/dev/ttyUSB0'
-MAINPORT = '/dev/ttyUSB0'
-BACKUPPORT = '/dev/ttyUSB1'
+DEFPORT = u'/dev/ttyUSB0'
+TIMYPORT = u'/dev/ttyUSB0'
+MAINPORT = u'/dev/ttyUSB0'
+BACKUPPORT = u'/dev/ttyUSB1'
+ENCODING = 'cp437'	# Timy serial interface encoding
 
 # TIMY serial baudrate
-#TIMY_BAUD = 9600	# Note: Timy cannot keep up with faster baud
 TIMY_BAUD = 38400	# 	but timy2 seems to cope ok :/
 
 # thread queue commands -> private to timy thread
@@ -160,18 +159,19 @@ def make_sectormap(layout=None):
                 
 def timy_checksum(msg):
     """Return the character sum for the Timy message string."""
-    return reduce(adder, msg, 0) & 0xff
+    return reduce(adder, msg, 0) & 0xff		# deprecated reduce function
 
 def timy_getsum(chkstr):
     """Convert Timy 'checksum' string to an integer."""
+    # assumes ord/string compat - ok for now
     return ((((ord(chkstr[0]) - 0x30) << 4) & 0xf0)
             | ((ord(chkstr[1]) - 0x30) & 0x0f))
 
 def chan2id(chanstr='0'):
     """Return a channel ID for the provided string, without fail."""
     ret = CHAN_UNKNOWN
-    if (type(chanstr) is str and len(chanstr) > 1
-        and chanstr[0] == 'C' and chanstr[1].isdigit()):
+    if (type(chanstr) in [unicode, str] and len(chanstr) > 1
+        and chanstr[0] == u'C' and chanstr[1].isdigit()):
         ret = int(chanstr[1])
     else:
         try:
@@ -184,14 +184,14 @@ def chan2id(chanstr='0'):
 
 def id2chan(chanid=0):
     """Return a normalised channel string for the provided channel id."""
-    ret = 'C?'
+    ret = u'C?'
     if type(chanid) is int and chanid >= CHAN_START and chanid <= CHAN_INT:
-        ret = 'C' + str(chanid)
+        ret = u'C' + unicode(chanid)
     return ret
 
 class timy(threading.Thread):
     """Timy thread object class."""
-    def __init__(self, port=None, name='timy'):
+    def __init__(self, port=None, name=u'timy'):
         """Construct timy thread object.
 
         Named parameters:
@@ -210,18 +210,18 @@ class timy(threading.Thread):
         self.lindex = 0
         self.clearing = False
         self.error = False
-        self.errstr = ''
+        self.errstr = u''
         self.cqueue = Queue.Queue()	# command queue
         self.log = logging.getLogger(self.name)
         self.log.setLevel(logging.DEBUG)
-        self.__rdbuf = ''
+        self.__rdbuf = ''	# should be bytestr?
         self.setcb()	# init but allow overwrite after loadconf
         if port is not None:
             self.setport(port)
 
     def __defcallback(self, evt=None):
         """Default callback is a tod log entry."""
-        self.log.debug(' ' + str(evt))
+        self.log.debug(u' ' + unicode(evt))
         return False
 
     def setcb(self, func=None):
@@ -232,9 +232,9 @@ class timy(threading.Thread):
         else:
             self.__cb = self.__defcallback
 
-    def printline(self, msg=''):
+    def printline(self, msg=u''):
         """Print msg to Timy printer, stripped and truncated."""
-        lmsg = str(msg).translate(strops.PRINT_TRANS)[0:32]
+        lmsg = msg[0:32]
         self.log.log(TIMER_LOG_LEVEL, lmsg)
         self.cqueue.put_nowait(('MSG', 'DTP' + lmsg + '\r'))
 
@@ -270,7 +270,7 @@ class timy(threading.Thread):
 
     def write(self, msg=None):
         """Queue a raw command string to attached Timy."""
-        self.cqueue.put_nowait(('MSG', str(msg).rstrip() + '\r'))
+        self.cqueue.put_nowait(('MSG', msg.rstrip() + '\r'))
 
     def exit(self, msg=None):
         """Request thread termination."""
@@ -384,10 +384,10 @@ class timy(threading.Thread):
                 if len(e) == 4:
                     ret = tod.tod(timeval = e[2], index = e[0], chan = e[1])
                 else:
-                    self.log.error('Invalid message: ' + repr(msg))
+                    self.log.error(u'Invalid message: ' + repr(msg))
             else:
-                self.log.error('Corrupt message: ' + repr(msg))
-                self.log.error('Checksum fail: 0x%02X != 0x%02X',
+                self.log.error(u'Corrupt message: ' + repr(msg))
+                self.log.error(u'Checksum fail: 0x%02X != 0x%02X',
                                tsum, csum)
         else:
             msg = msg.strip()
@@ -415,7 +415,7 @@ class timy(threading.Thread):
         channo = chan2id(st.chan)
         if channo != CHAN_UNKNOWN:
             if self.arms[channo]:
-                #self.log.debug('Queueing ToD: ' + str(st))
+                #self.log.debug(u'Queueing ToD: ' + str(st))
                 glib.idle_add(self.__cb, st)
                 if not self.armlocked:
                     self.arms[channo] = False
@@ -430,14 +430,14 @@ class timy(threading.Thread):
                 if index > 2000 and not self.clearing:
                     self.clearing = True
                     self.clrmem()
-                    self.log.log(TIMER_LOG_LEVEL, '-- auto clear memory --')
+                    self.log.log(TIMER_LOG_LEVEL, u'-- auto clear memory --')
         else:
             pass # ok to ignore these messages
         return False
 
     def __read(self):
         """Read messages from timy until a timeout condition."""
-        ch = self.port.read(1)
+        ch = self.port.read(1)	# decode via cp437?
         while ch != '':
             if ch == CR:
                 # Return ends the current 'message'
@@ -453,7 +453,7 @@ class timy(threading.Thread):
     def run(self):
         """Called via threading.Thread.start()."""
         running = True
-        self.log.debug('Starting')
+        self.log.debug(u'Starting')
         while running:
             try:
                 # Read phase
@@ -468,16 +468,15 @@ class timy(threading.Thread):
                 # Write phase
                 if type(m) is tuple and type(m[0]) is str and m[0] in TCMDS:
                     if m[0] == 'MSG' and self.port is not None and not self.error:
-                        cmd = m[1]
-                        self.log.debug('Sending rawmsg ' + repr(str(m[1])))
-                        self.port.write(cmd)
+                        self.log.debug(u'Sending rawmsg ' + repr(m[1]))
+                        self.port.write(m[1].encode(ENCODING,'replace'))
                     elif m[0] == 'ARM':
                         chan = chan2id(m[1])
-                        self.log.debug('Arming channel ' + id2chan(chan))
+                        self.log.debug(u'Arming channel ' + id2chan(chan))
                         self.arms[chan] = True;
                     elif m[0] == 'DEARM':
                         chan = chan2id(m[1])
-                        self.log.debug('Disarming channel ' + id2chan(chan))
+                        self.log.debug(u'Disarming channel ' + id2chan(chan))
                         self.arms[chan] = False;
                     elif m[0] == 'TRIG':
                         if type(m[1]) is tod.tod:
@@ -485,24 +484,24 @@ class timy(threading.Thread):
                     elif m[0] == 'RCLR':
                         self.clearing = False
                     elif m[0] == 'EXIT':
-                        self.log.debug('Request to close : ' + str(m[1]))
+                        self.log.debug(u'Request to close : ' + str(m[1]))
                         running = False	# This may already be set
                     elif m[0] == 'PORT':
                         if self.port is not None:
                             self.port.close()
                             self.port = None
                         if m[1] is not None and m[1] != '' and m[1] != 'NULL':
-                            self.log.debug('Re-Connect port : ' + str(m[1]))
-                            self.port = serial.Serial(m[1], TIMY_BAUD, timeout=0.2)
-                                                      #rtscts=1, timeout=0.2)
+                            self.log.debug(u'Re-Connect port: ' + repr(m[1]))
+                            self.port = serial.Serial(m[1], TIMY_BAUD,
+                                                      rtscts=1, timeout=0.2)
                             self.error = False
                         else:
-                            self.log.debug('Not connected.')
+                            self.log.debug(u'Not connected.')
                             self.error = True
                     else:
                         pass
                 else:
-                    self.log.warn('Unknown message: ' + repr(m))
+                    self.log.warn(u'Unknown message: ' + repr(m))
             except Queue.Empty:
                 pass
             except serial.SerialException as e:
@@ -517,7 +516,6 @@ class timy(threading.Thread):
                 self.errstr = str(e)
                 self.error = True
         if self.port is not None:
-            # TODO: closing real serial port locks up here... sleuth rqd
             self.port.close()
             self.port = None
         self.log.info('Exiting')
@@ -534,6 +532,7 @@ if __name__ == "__main__":
     lh.setFormatter(logging.Formatter(
                       "%(asctime)s %(levelname)s:%(name)s: %(message)s"))
     t.log.addHandler(lh)
+    ucnt = 0
     try:
         t.start()
         t.sane()
@@ -542,6 +541,13 @@ if __name__ == "__main__":
         t.arm(0)
         t.arm(1)
         t.arm(2)
+        #while ucnt < 0x300:
+            #tmsg = u'{0:03x} :: '.format(ucnt)
+            #for j in xrange(ucnt, ucnt+16):
+                #tmsg += unichr(j)
+            #t.printline(tmsg)
+            #ucnt+=16
+            #time.sleep(0.2)
         gtk.main()
     except:
         t.exit('Exception')
