@@ -431,8 +431,9 @@ class agg(tod):
                                 str(type(other).__name__)))
 
 # ToD 'constants'
-ZERO = tod()
-MAX = tod(u'23h59:59.9999')
+ZERO = tod()			# common cases
+ONE = tod(u'1.0')
+MAX = tod(u'23h59:59.9999')	# largest val possible
 
 # Fake times for special cases
 # these are impossible tods that still sort correctly
