@@ -49,6 +49,8 @@ USCBSRV_HOST=''		# default is "not present"
 USCBSRV_PORT=6667
 USCBSRV_CHANNEL='#announce'
 USCBSRV_SRVNICK='uSCBsrv'
+CMDMAX = 400	# allow for up to 100 in header... not a good solution!
+		# irc layer requires multi-line handling
 
 # dispatch thread queue commands
 TCMDS = ('EXIT', 'PORT', 'MSG')
@@ -111,6 +113,14 @@ class uscbsrv(threading.Thread):
         """Clear the specified line in DHI database."""
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),erl=True))
 
+    def send_cmd(self, hdr=None, txt=None):
+        """Send the provided new style command."""
+        if txt and len(txt) > CMDMAX:
+            self.log.debug('Truncated long message.')
+            txt = strops.truncpad(txt, CMDMAX, elipsis=False)
+        self.sendmsg(unt4.unt4(header=hdr,text=txt))
+## TODO: resolve all the announce/command/channel inconsistencies
+
     def set_title(self, line):
         """Update the announcer's title line."""
         self.sendmsg(unt4.unt4(header='title',text=line))
@@ -240,7 +250,8 @@ class uscbsrv(threading.Thread):
             self.log.debug('Joined channel ' + str(e.target()))
             self.dumbcnt = 0
         else:
-            self.log.debug('Joined additional channel ' + str(e.target()))
+            self.log.debug(u'Channel join: ' + repr(su)
+                             + u'/' + repr(e.target()))
 
     def channel_part_cb(self, c, e):
         """Register channel part."""
@@ -256,18 +267,27 @@ class uscbsrv(threading.Thread):
 
     def pubmsg_cb(self, c, e):
         """Handle message in channel."""
+        ## TODO: this should track u4 buffers on a per-nick basis
+        ##       otherwise interleaved commands will be corrupted
+        ##       but ok while one u4 per line and only a few commanders
+        ##       correct the u4 buffer problems: REWRITE FOR JSON
         su = self.il.nm_to_n(e.source()).lower()
         tg = e.target().lower()
         if tg == self.channel:
-            self.log.debug('Public message from ' + su)
+            #self.log.debug('Public message from ' + su)
             if self.pub_cb is not None:
                 self.pub_rdbuf += unt4.decode(''.join(e.arguments()))
-                idx = self.pub_rdbuf.find(chr(unt4.EOT))
-                if idx >= 0:
-                    msgtxt = self.pub_rdbuf[0:idx+1]
-                    self.pub_rdbuf = self.pub_rdbuf[idx+1:]
-                    glib.idle_add(self.pub_cb, unt4.unt4(unt4str=msgtxt.decode('utf-8','replace')))
+                oidx = self.pub_rdbuf.find(chr(unt4.SOH))
+                eidx = self.pub_rdbuf.find(chr(unt4.EOT))
+                if oidx >= 0 and eidx >= 0 and eidx > oidx:
+                    msgtxt = self.pub_rdbuf[oidx:eidx+1]
+                    self.pub_rdbuf = self.pub_rdbuf[eidx+1:]
+                    glib.idle_add(self.pub_cb,
+                        unt4.unt4(unt4str=msgtxt.decode('utf-8','replace')),
+                        su)
                     self._pacing(0.0)
+                elif eidx >= 0:	# discard partial message (not complete)
+                    self.pub_rdbuf = self.pub_rdbuf[eidx+1:]
         
     def privmsg_cb(self, c, e):
         """Handle private message."""
